@@ -7,27 +7,17 @@ from os import path
 ptype = {'scipy': ScipyParticle, 'jit': JITParticle}
 
 
-def set_ofam_fieldset():
+def set_ofam_fieldset(full_load=False):
     filenames = {'U': path.join(path.dirname(__file__), 'OFAM_example_data', 'OFAM_simple_U.nc'),
                  'V': path.join(path.dirname(__file__), 'OFAM_example_data', 'OFAM_simple_V.nc')}
     variables = {'U': 'u', 'V': 'v'}
     dimensions = {'lat': 'yu_ocean', 'lon': 'xu_ocean', 'depth': 'st_ocean',
                   'time': 'Time'}
-    return FieldSet.from_netcdf(filenames, variables, dimensions, allow_time_extrapolation=True)
-
-
-def test_ofam_fieldset():
-    fieldset = set_ofam_fieldset()
-    assert(fieldset.U.lon.size == 2001)
-    assert(fieldset.U.lat.size == 601)
-    assert(fieldset.U.data.shape == (4, 601, 2001))
-    assert(fieldset.V.lon.size == 2001)
-    assert(fieldset.V.lat.size == 601)
-    assert(fieldset.V.data.shape == (4, 601, 2001))
+    return FieldSet.from_netcdf(filenames, variables, dimensions, allow_time_extrapolation=True, full_load=full_load)
 
 
 def test_ofam_fieldset_fillvalues():
-    fieldset = set_ofam_fieldset()
+    fieldset = set_ofam_fieldset(full_load=True)
     # V.data[0, 0, 150] is a landpoint, that makes NetCDF4 generate a masked array, instead of an ndarray
     assert(fieldset.V.data[0, 0, 150] == 0)
 
